@@ -82,6 +82,11 @@ sigchld_handler (int sig)
     lastsig = SIGCHLD;
 }
 
+/* 
+ * This function is called by other parts of the program to
+ * setup the sigchld handler after a change to the signal context.
+ * This is done to improve robustness of the signal handling code.
+ */
 void deal_with_sigchld(void)
 {
   RETSIGTYPE sigchld_handler(int);
@@ -90,10 +95,13 @@ void deal_with_sigchld(void)
 
   memset (&sa_new, 0, sizeof sa_new);
   sigemptyset (&sa_new.sa_mask);
-  sa_new.sa_handler = SIG_IGN;
+  /* sa_new.sa_handler = SIG_IGN;     pointless */
 
   /* set up to catch child process termination signals */ 
   sa_new.sa_handler = sigchld_handler;
+#ifdef SA_RESTART	/* SunOS 4.1 portability hack */
+  sa_new.sa_flags = SA_RESTART | SA_NOCLDSTOP;
+#endif
   sigaction (SIGCHLD, &sa_new, NULL);
 #if defined(SIGPWR)
   sigaction (SIGPWR, &sa_new, NULL);
@@ -127,6 +135,9 @@ daemonize (const char *logfile, void (*termhook)(int))
   memset (&sa_new, 0, sizeof sa_new);
   sigemptyset (&sa_new.sa_mask);
   sa_new.sa_handler = SIG_IGN;
+#ifdef SA_RESTART	/* SunOS 4.1 portability hack */
+  sa_new.sa_flags = SA_RESTART;
+#endif
 #endif /* HAVE_SIGACTION */
 #ifdef 	SIGTTOU
 #ifndef HAVE_SIGACTION
