@@ -40,33 +40,12 @@
 #include "mx.h"
 #endif /* HAVE_RES_SEARCH */
 
+#include "kerberos.h"
 #ifdef KERBEROS_V4
-#ifdef KERBEROS_V5
-#include <kerberosIV/des.h>
-#include <kerberosIV/krb.h>
-#else
-#if defined (__bsdi__)
-#include <des.h> /* order of includes matters */
-#include <krb.h>
-#define krb_get_err_text(e) (krb_err_txt[e])
-#else
-#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__linux__)
-#define krb_get_err_text(e) (krb_err_txt[e])
-#include <krb.h>
-#include <des.h>
-#else
-#include <krb.h>
-#include <des.h>
-#endif /* ! defined (__FreeBSD__) */
-#endif /* ! defined (__bsdi__) */
-#endif /* KERBEROS_V5 */
 #include <netinet/in.h>
 #include <netdb.h>
 #endif /* KERBEROS_V4 */
-#ifdef KERBEROS_V5
-#include <krb5.h>
-#include <com_err.h>
-#endif /* KERBEROS_V5 */
+
 #include "i18n.h"
 
 #include "socket.h"
@@ -88,6 +67,7 @@ int stage;		/* where are we? */
 int phase;		/* where are we, for error-logging purposes? */
 int mytimeout;		/* value of nonreponse timeout */
 int suppress_tags;	/* emit tags? */
+char shroud[PASSWORDLEN];	/* string to shroud in debug output */
 
 static const struct method *protocol;
 static jmp_buf	restart;
@@ -96,7 +76,6 @@ char tag[TAGLEN];
 static int tagnum;
 #define GENSYM	(sprintf(tag, "A%04d", ++tagnum % TAGMOD), tag)
 
-static char shroud[PASSWORDLEN];	/* string to shroud in debug output */
 static int timeoutcount;		/* count consecutive timeouts */
 static int msglen;			/* actual message length */
 
@@ -781,9 +760,27 @@ static int readheaders(int sock,
 	    resent_from_offs = (line - msgblk.headers);
 	else if (!strncasecmp("Apparently-From:", line, 16))
 	    app_from_offs = (line - msgblk.headers);
-	else if (!strncasecmp("Sender:", line, 7))
+	/*
+	 * Netscape 4.7 puts "Sender: zap" in mail headers.  Perverse...
+	 *
+	 * But a literal reading of RFC822 sec. 4.4.2 supports the idea
+	 * that Sender: *doesn't* have to be a working email address.
+	 *
+	 * The definition of the Sender header in RFC822 says, in
+	 * part, "The Sender mailbox specification includes a word
+	 * sequence which must correspond to a specific agent (i.e., a
+	 * human user or a computer program) rather than a standard
+	 * address."  That implies that the contents of the Sender
+	 * field don't need to be a legal email address at all So
+	 * ignore any Sender or Resent-Semnder lines unless they
+	 * contain @.
+	 *
+	 * (RFC2822 says the condents of Sender must be a valid mailbox
+	 * address, which is also what RFC822 4.4.4 implies.)
+	 */
+	else if (!strncasecmp("Sender:", line, 7) && strchr(line, '@'))
 	    sender_offs = (line - msgblk.headers);
-	else if (!strncasecmp("Resent-Sender:", line, 14))
+	else if (!strncasecmp("Resent-Sender:", line, 14) && strchr(line, '@'))
 	    resent_sender_offs = (line - msgblk.headers);
 
 #ifdef __UNUSED__
@@ -1703,7 +1700,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	    int err_no = errno;
 #ifdef HAVE_RES_SEARCH
 	    if (err_no != 0 && h_errno != 0)
-		report(stderr, _("fetchmail: internal inconsistency\n"));
+		report(stderr, _("internal inconsistency\n"));
 #endif
 	    /*
 	     * Avoid generating a bogus error every poll cycle when we're
@@ -1713,7 +1710,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	    if (!((err_no == EHOSTUNREACH || err_no == ENETUNREACH) 
 		  && run.poll_interval))
 	    {
-		report_build(stderr, _("fetchmail: %s connection to %s failed"), 
+		report_build(stderr, _("%s connection to %s failed"), 
 			     protocol->name, ctl->server.pollname);
 #ifdef HAVE_RES_SEARCH
 		if (h_errno != 0)
@@ -1769,7 +1766,8 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	/* perform initial SSL handshake on open connection */
 	/* Note:  We pass the realhost name over for certificate
 		verification.  We may want to make this configurable */
-	if (ctl->use_ssl && SSLOpen(mailserver_socket,ctl->sslkey,ctl->sslcert,realhost) == -1) 
+	if (ctl->use_ssl && SSLOpen(mailserver_socket,ctl->sslkey,ctl->sslcert,ctl->sslproto,ctl->sslcertck,
+	    ctl->sslcertpath,ctl->sslfingerprint,realhost,ctl->server.pollname) == -1) 
 	{
 	    report(stderr, _("SSL connection failed.\n"));
 	    goto closeUp;
@@ -1808,44 +1806,67 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	stage = STAGE_GETAUTH;
 	if (protocol->getauth)
 	{
-	    if (protocol->password_canonify)
-		(protocol->password_canonify)(shroud, ctl->password, PASSWORDLEN);
-	    else
-		strcpy(shroud, ctl->password);
-
 	    ok = (protocol->getauth)(mailserver_socket, ctl, buf);
+
 	    if (ok != 0)
 	    {
 		if (ok == PS_LOCKBUSY)
 		    report(stderr, _("Lock-busy error on %s@%s\n"),
 			  ctl->remotename,
 			  ctl->server.truename);
-		else if (ok == PS_AUTHFAIL)
-		{
-		    report(stderr, _("Authorization failure on %s@%s\n"), 
+		else if (ok == PS_SERVBUSY)
+		    report(stderr, _("Server busy error on %s@%s\n"),
 			  ctl->remotename,
 			  ctl->server.truename);
+		else if (ok == PS_AUTHFAIL)
+		{
+		    report(stderr, _("Authorization failure on %s@%s%s\n"), 
+			   ctl->remotename,
+			   ctl->server.truename,
+			   (ctl->wehaveauthed ? " (previously authorized)" : " ")
+			);
 
 		    /*
 		     * If we're running in background, try to mail the
 		     * calling user a heads-up about the authentication 
 		     * failure once it looks like this isn't a fluke 
 		     * due to the server being temporarily inaccessible.
-		     * When we get third failure, we notify the user.  After
-		     * that, once we get authorization we let the user know
-		     * service is restored.
+		     * When we get third succesive failure, we notify the user
+		     * but only if we haven't already managed to get
+		     * authorization.  After that, once we get authorization
+		     * we let the user know service is restored.
 		     */
 		    if (run.poll_interval
-			&& ++ctl->authfailcount == 3
+			&& ctl->wehavesentauthnote
+			&& ((ctl->wehaveauthed && ++ctl->authfailcount == 10)
+			    || ++ctl->authfailcount == 3)
 			&& !open_warning_by_mail(ctl, (struct msgblk *)NULL))
 		    {
+			ctl->wehavesentauthnote = 1;
 			stuff_warning(ctl,
-			    _("Subject: fetchmail authentication failed\r\n"));
+				      _("Subject: fetchmail authentication failed on %s@%s\r\n"),
+			    ctl->remotename, ctl->server.truename);
 			stuff_warning(ctl,
-			    _("Fetchmail could not get mail from %s@%s.\r\n"), 
-			    ctl->remotename,
-			    ctl->server.truename);
-			stuff_warning(ctl, _("\
+				      _("Fetchmail could not get mail from %s@%s.\r\n"), 
+				      ctl->remotename,
+				      ctl->server.truename);
+			if (ctl->wehaveauthed)
+			    stuff_warning(ctl, _("\
+The attempt to get authorization failed.\r\n\
+Since we have already succeeded in getting authorization for this\r\n\
+connection, this is probably another failure mode (such as busy server)\r\n\
+that fetchmail cannot distinguish because the server didn't send a useful\r\n\
+error message.\r\n\
+\r\n\
+However, if you HAVE changed you account details since starting the\r\n\
+fetchmail daemon, you need to stop the daemon, change your configuration\r\n\
+of fetchmail, and then restart the daemon.\r\n\
+\r\n\
+The fetchmail daemon will continue running and attempt to connect\r\n\
+at each cycle.  No future notifications will be sent until service\r\n\
+is restored."));
+			else
+			    stuff_warning(ctl, _("\
 The attempt to get authorization failed.\r\n\
 This probably means your password is invalid, but some servers have\r\n\
 other failure modes that fetchmail cannot distinguish from this\r\n\
@@ -1859,15 +1880,28 @@ is restored."));
 		}
 		else
 		    report(stderr, _("Unknown login or authentication error on %s@%s\n"),
-			  ctl->remotename,
-			  ctl->server.truename);
+			   ctl->remotename,
+			   ctl->server.truename);
 		    
 		goto cleanUp;
 	    }
 	    else
 	    {
-		if (ctl->authfailcount >= 3)
+		/*
+		 * This connection has given us authorization at least once.
+		 *
+		 * There are dodgy server (clubinternet.fr for example) that
+		 * give spurious authorization failures on patently good
+		 * account/password details, then 5 minutes later let you in!
+		 *
+		 * This is meant to build in some tolerance of such nasty bits
+		 * of work.
+		 */
+		ctl->wehaveauthed = 1;
+		/*if (ctl->authfailcount >= 3)*/
+		if (ctl->wehavesentauthnote)
 		{
+		    ctl->wehavesentauthnote = 0;
 		    report(stderr,
 			   _("Authorization OK on %s@%s\n"),
 			   ctl->remotename,
@@ -1875,18 +1909,25 @@ is restored."));
 		    if (!open_warning_by_mail(ctl, (struct msgblk *)NULL))
 		    {
 			stuff_warning(ctl,
-			    _("Subject: fetchmail authentication OK\r\n"));
+			      _("Subject: fetchmail authentication OK on %s@%s\r\n"),
+				      ctl->remotename, ctl->server.truename);
 			stuff_warning(ctl,
-			    _("Fetchmail was able to log into %s@%s.\r\n"), 
-			    ctl->remotename,
-			    ctl->server.truename);
+			      _("Fetchmail was able to log into %s@%s.\r\n"), 
+				      ctl->remotename,
+				      ctl->server.truename);
 			stuff_warning(ctl, 
-			    _("Service has been restored.\r\n"));
+				      _("Service has been restored.\r\n"));
 			close_warning_by_mail(ctl, (struct msgblk *)NULL);
 		    
 		    }
-		    ctl->authfailcount = 0;
 		}
+		/*
+		 * Reporting only after the first three
+		 * consecutive failures, or ten consecutive
+		 * failures after we have managed to get
+		 * authorization.
+		 */
+		ctl->authfailcount = 0;
 	    }
 	}
 
@@ -2306,8 +2347,11 @@ is restored."));
 
 			    for (sdp = ctl->newsaved; sdp; sdp = sdp->next)
 				if ((sdp->val.status.num == num)
-						&& (!toolarge || oldmsg))
+						&& (!toolarge || oldmsg)) 
+				{
 				    sdp->val.status.mark = UID_SEEN;
+				    save_str(&ctl->oldsaved, sdp->id,UID_SEEN);
+				}
 			}
 
 			/* maybe we delete this message now? */
@@ -2427,7 +2471,7 @@ is restored."));
     }
 
 closeUp:
-    /* execute post-initialization command, if any */
+    /* execute wrapup command, if any */
     if (ctl->postconnect && (ok = system(ctl->postconnect)))
     {
 	report(stderr, _("post-connection command failed with status %d\n"), ok);
@@ -2570,7 +2614,7 @@ va_dcl
     {
 	char *cp;
 
-	if (shroud && shroud[0] && (cp = strstr(buf, shroud)))
+	if (shroud[0] && (cp = strstr(buf, shroud)))
 	{
 	    char	*sp;
 
