@@ -80,8 +80,6 @@ void *request = NULL;
 int requestlen = 0;
 #endif /* NET_SECURITY */
 
-static char *lockfile;		/* name of lockfile */
-static int lock_acquired;	/* have we acquired a lock */
 static int querystatus;		/* status of query */
 static int successes;		/* count number of successful polls */
 static int activecount;		/* count number of active entries */
@@ -91,17 +89,6 @@ static time_t parsetime;	/* time of last parse */
 static void terminate_run(int);
 static void terminate_poll(int);
 
-#ifdef HAVE_ON_EXIT
-static void unlockit(int n, void *p)
-#else
-static void unlockit(void)
-#endif
-/* must-do actions for exit (but we can't count on being able to do malloc) */
-{
-    if (lockfile && lock_acquired)
-	unlink(lockfile);
-}
-
 #if defined(__FreeBSD__) && defined(__FreeBSD_USE_KVM)
 /* drop SGID kmem privileage until we need it */
 static void dropprivs(void)
@@ -145,7 +132,6 @@ int main(int argc, char **argv)
 {
     int st, bkgd = FALSE;
     int parsestatus, implicitmode = FALSE;
-    FILE	*lockfp;
     struct query *ctl;
     netrc_entry *netrc_list;
     char *netrc_file, *tmpbuf;
@@ -187,19 +173,14 @@ int main(int argc, char **argv)
     outlevel = O_NORMAL;
 
     /*
-     * We used to arrange for the lockfile to be removed on exit close
+     * We used to arrange for the lock to be removed on exit close
      * to where the lock was asserted.  Now we need to do it here, because
-     * we might have re-executed in background with an existing lockfile
+     * we might have re-executed in background with an existing lock
      * as the result of a changed rcfile (see the code near the execvp(3)
      * call near the beginning of the polling loop for details).  We want
-     * to be sure the lockfile gets nuked on any error exit, basically.
+     * to be sure the lock gets nuked on any error exit, basically.
      */
-#ifdef HAVE_ATEXIT
-    atexit(unlockit);
-#endif
-#ifdef HAVE_ON_EXIT
-    on_exit(unlockit, (char *)NULL);
-#endif
+    lock_dispose();
 
     if ((parsestatus = parsecmdline(argc,argv, &cmd_run, &cmd_opts)) < 0)
 	exit(PS_SYNTAX);
@@ -279,21 +260,8 @@ int main(int argc, char **argv)
 #endif
 	report_init((run.poll_interval == 0 || nodetach) && !run.logfile);
 
-    /* set up to do lock protocol */
-#define	FETCHMAIL_PIDFILE	"fetchmail.pid"
-    if (!getuid()) {
-	xalloca(tmpbuf, char *,
-		sizeof(PID_DIR) + sizeof(FETCHMAIL_PIDFILE));
-	sprintf(tmpbuf, "%s/%s", PID_DIR, FETCHMAIL_PIDFILE);
-    } else {
-	xalloca(tmpbuf, char *, strlen(fmhome) + sizeof(FETCHMAIL_PIDFILE) + 2);
-	strcpy(tmpbuf, fmhome);
-	strcat(tmpbuf, "/");
-        if (fmhome == home)
- 	   strcat(tmpbuf, ".");
-	strcat(tmpbuf, FETCHMAIL_PIDFILE);
-    }
-#undef FETCHMAIL_PIDFILE
+    /* construct the lockfile */
+    lock_setup();
 
 #ifdef HAVE_SETRLIMIT
     /*
@@ -360,9 +328,6 @@ int main(int argc, char **argv)
 				havercfile ? "" :  _(" and "),
 				havercfile ? "" : rcfile);
 
-	if (outlevel >= O_VERBOSE)
-	    printf(_("Lockfile at %s\n"), tmpbuf);
-
 	if (querylist == NULL)
 	    fprintf(stderr,
 		    _("No mailservers set up -- perhaps %s is missing?\n"),
@@ -380,29 +345,12 @@ int main(int argc, char **argv)
     }
 
     /* check for another fetchmail running concurrently */
-    pid = -1;
-    if ((lockfile = (char *) malloc(strlen(tmpbuf) + 1)) == NULL)
-    {
-	report(stderr,_("fetchmail: cannot allocate memory for lock name.\n"));
-	exit(PS_EXCLUDE);
-    }
-    else
-	(void) strcpy(lockfile, tmpbuf);
-    if ((lockfp = fopen(lockfile, "r")) != NULL )
-    {
-	bkgd = (fscanf(lockfp,"%d %d", &pid, &st) == 2);
-
-	if (kill(pid, 0) == -1) {
-	    fprintf(stderr,_("fetchmail: removing stale lockfile\n"));
-	    pid = -1;
-	    bkgd = FALSE;
-	    unlink(lockfile);
-	}
-	fclose(lockfp);	/* not checking should be safe, file mode was "r" */
-    }
+    pid = lock_state();
+    bkgd = (pid < 0);
+    pid = bkgd ? -pid : pid;
 
     /* if no mail servers listed and nothing in background, we're done */
-    if (!(quitmode && argc == 2) && pid == -1 && querylist == NULL) {
+    if (!(quitmode && argc == 2) && pid == 0 && querylist == NULL) {
 	(void)fputs(_("fetchmail: no mailservers have been specified.\n"),stderr);
 	exit(PS_SYNTAX);
     }
@@ -410,7 +358,7 @@ int main(int argc, char **argv)
     /* perhaps user asked us to kill the other fetchmail */
     if (quitmode)
     {
-	if (pid == -1) 
+	if (pid == 0) 
 	{
 	    fprintf(stderr,_("fetchmail: no other fetchmail is running\n"));
 	    if (argc == 2)
@@ -426,16 +374,16 @@ int main(int argc, char **argv)
 	{
 	    fprintf(stderr,_("fetchmail: %s fetchmail at %d killed.\n"),
 		    bkgd ? _("background") : _("foreground"), pid);
-	    unlink(lockfile);
+	    lock_release();
 	    if (argc == 2)
 		exit(0);
 	    else
-		pid = -1; 
+		pid = 0; 
 	}
     }
 
     /* another fetchmail is running -- wake it up or die */
-    if (pid != -1)
+    if (pid != 0)
     {
 	if (check_only)
 	{
@@ -461,7 +409,7 @@ int main(int argc, char **argv)
 	{
 	    /* this test enables re-execing on a changed rcfile */
 	    if (getpid() == pid)
-		lock_acquired = TRUE;
+		lock_assert();
 	    else
 	    {
 		fprintf(stderr,
@@ -566,29 +514,7 @@ int main(int argc, char **argv)
     signal(SIGQUIT, terminate_run);
 
     /* here's the exclusion lock */
-#ifndef O_SYNC
-#define O_SYNC	0	/* use it if we have it */
-#endif
-    if (!lock_acquired)
-    {
-      if ((st = open(lockfile, O_WRONLY|O_CREAT|O_EXCL|O_SYNC, 0666)) != -1)
-      {
-	  sprintf(tmpbuf,"%d", getpid());
-	  write(st, tmpbuf, strlen(tmpbuf));
-	  if (run.poll_interval)
-	  {
-	      sprintf(tmpbuf," %d", run.poll_interval);
-	      write(st, tmpbuf, strlen(tmpbuf));
-	  }
-	  close(st);	/* should be safe, fd was opened with O_SYNC */
-	  lock_acquired = TRUE;
-      }
-      else
-      {
-	  fprintf(stderr,	_("fetchmail: lock creation failed.\n"));
-	  return(PS_EXCLUDE);
-      }
-    }
+    lock_or_die();
 
     /*
      * Query all hosts. If there's only one, the error return will
@@ -916,6 +842,10 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(use_ssl);
     FLAG_MERGE(sslkey);
     FLAG_MERGE(sslcert);
+    FLAG_MERGE(sslproto);
+    FLAG_MERGE(sslcertck);
+    FLAG_MERGE(sslcertpath);
+    FLAG_MERGE(sslfingerprint);
 #endif
     FLAG_MERGE(expunge);
 
@@ -1049,6 +979,30 @@ static int load_params(int argc, char **argv, int optind)
     if (check_only && run.poll_interval)
 	run.poll_interval = 0;
 
+    /*
+     * DNS support is required for some protocols.  We used to
+     * do this unconditionally, but it made fetchmail excessively
+     * vulnerable to misconfigured DNS setups.
+     *
+     * If we're using ETRN or ODMR, the smtp hunt list is the
+     * list of systems we're polling on behalf of; these have
+     * to be fully-qualified domain names.  The default for
+     * this list should be the FQDN of localhost.
+     *
+     * If we're using Kerberos for authentication, we need 
+     * the FQDN in order to generate capability keys.
+     */
+    if (strcmp(fetchmailhost, "localhost") == 0)
+	for (ctl = querylist; ctl; ctl = ctl->next)
+	    if (ctl->active && 
+		(ctl->server.protocol==P_ETRN || ctl->server.protocol==P_ODMR
+		 || ctl->server.authenticate == A_KERBEROS_V4
+		 || ctl->server.authenticate == A_KERBEROS_V5))
+	    {
+		fetchmailhost = host_fqdn();
+		break;
+	    }
+
     /* merge in wired defaults, do sanity checks and prepare internal fields */
     for (ctl = querylist; ctl; ctl = ctl->next)
     {
@@ -1084,28 +1038,11 @@ static int load_params(int argc, char **argv, int optind)
 	    DEFAULT(ctl->server.uidl, FALSE);
 #ifdef	SSL_ENABLE
 	    DEFAULT(ctl->use_ssl, FALSE);
+	    DEFAULT(ctl->sslcertck, FALSE);
 #endif
 	    DEFAULT(ctl->server.checkalias, FALSE);
 #undef DEFAULT
 
-	    /*
-	     * DNS support is required for some protocols.  We used to
-	     * do this unconditionally, but it made fetchmail excessively
-	     * vulnerable to misconfigured DNS setups.
-	     *
-	     * If we're using ETRN or ODMR, the smtp hunt list is the
-	     * list of systems we're polling on behalf of; these have
-	     * to be fully-qualified domain names.  The default for
-	     * this list should be the FQDN of localhost.
-	     *
-	     * If we're using Kerberos for authentication, we need 
-	     * the FQDN in order to generate capability keys.  */
-	    if (ctl->server.protocol==P_ETRN || ctl->server.protocol==P_ODMR
-		|| ctl->server.authenticate == A_KERBEROS_V4
-		|| ctl->server.authenticate == A_KERBEROS_V5)
-		if (strcmp(fetchmailhost, "localhost") == 0)
-			fetchmailhost = host_fqdn();
-
 	    /*
 	     * Make sure we have a nonempty host list to forward to.
 	     */
@@ -1129,23 +1066,7 @@ static int load_params(int argc, char **argv, int optind)
 	    }
 #endif /* !HAVE_GETHOSTBYNAME || !HAVE_RES_SEARCH */
 
-	    /*
-	     *
-	     * Compute the true name of the mailserver host.  
-	     * There are two clashing cases here:
-	     *
-	     * (1) The poll name is a label, possibly on one of several
-	     *     poll configurations for the same host.  In this case 
-	     *     the `via' option will be present and give the true name.
-	     *
-	     * (2) The poll name is the true one, the via name is 
-	     *     localhost.   This is going to be typical for ssh-using
-	     *     configurations.
-	     *
-	     * We're going to assume the via name is true unless it's
-	     * localhost.
-	     */
-	    if (ctl->server.via && strcmp(ctl->server.via, "localhost"))
+	    if (ctl->server.via) 
 		ctl->server.queryname = xstrdup(ctl->server.via);
 	    else
 		ctl->server.queryname = xstrdup(ctl->server.pollname);
@@ -1190,7 +1111,7 @@ static int load_params(int argc, char **argv, int optind)
 		ctl->server.truename = xstrdup(leadname);
 	    }
 #ifdef HAVE_GETHOSTBYNAME
-	    else if (!configdump)
+	    else if (ctl->active && !configdump)
 	    {
 		if (ctl->server.authenticate==A_KERBEROS_V4 ||
 		      ctl->server.authenticate==A_KERBEROS_V5 ||
@@ -1213,21 +1134,23 @@ static int load_params(int argc, char **argv, int optind)
 			ctl->server.truename=xstrdup((char *)namerec->h_name);
 		}
 #endif /* HAVE_GETHOSTBYNAME */
-		else {
+		else if (ctl->active)
+		{
 #ifdef HAVE_GETHOSTBYNAME
 		    struct hostent	*namerec;
 		    
-		    /* <fetchmail@mail.julianhaight.com>
-		       Get the host's IP, so we can report it like this:
-
-		       Received: from hostname [10.0.0.1]
-
-		       do we actually need to gethostbyname to find the IP?
-		       it seems like it would be faster to do this later, when
-		       we are actually resolving the hostname for a connection,
-		       but I ain't that smart, so I don't know where to make
-		       the change later..
-		    */
+		    /* 
+		     * Get the host's IP, so we can report it like this:
+		     *
+		     * Received: from hostname [10.0.0.1]
+		     *
+		     * For ultra-efficiency, we should find the IP later, when
+		     * we are actually resolving the hostname for a connection.
+		     * Problem is this would have to be done inside SockOpen
+		     * and there's no way to do that that wouldn't both (a)
+		     * be horribly complicated, and (b) blow a couple of
+		     * layers of modularity all to hell.
+		     */
 		    errno = 0;
 		    namerec = gethostbyname(ctl->server.queryname);
 		    if (namerec == (struct hostent *)NULL)
@@ -1249,6 +1172,12 @@ static int load_params(int argc, char **argv, int optind)
 #endif /* HAVE_GETHOSTBYNAME */
 		}
 	    }
+	    else
+		/*
+		 * This is a kluge.  It enables users to edit their
+		 * configurations when DNS isn't available.
+		 */
+		ctl->server.truename = xstrdup(ctl->server.queryname);
 
 	    /* if no folders were specified, set up the null one as default */
 	    if (!ctl->mailboxes)
@@ -1400,7 +1329,7 @@ static void terminate_run(int sig)
 	  memset(ctl->password, '\0', strlen(ctl->password));
 
 #if !defined(HAVE_ATEXIT) && !defined(HAVE_ON_EXIT)
-    unlockit();
+    lock_release();
 #endif
 
     if (activecount == 0)
@@ -1471,9 +1400,6 @@ static int query_host(struct query *ctl)
 #endif /* POP3_ENABLE */
 	break;
     case P_IMAP:
-#ifdef GSSAPI
-    case P_IMAP_GSS:
-#endif /* GSSAPI */
 #ifdef IMAP_ENABLE
 	return(doIMAP(ctl));
 #else
@@ -1637,7 +1563,14 @@ static void dump_params (struct runctl *runp,
 	    printf(_("  Mail service principal is: %s\n"), ctl->server.principal);
 #ifdef	SSL_ENABLE
 	if (ctl->use_ssl)
-	    printf("  SSL encrypted sessions enabled.\n");
+	    printf(_("  SSL encrypted sessions enabled.\n"));
+	if (ctl->sslcertck) {
+	    printf(_("  SSL server certificate checking enabled.\n"));
+	    if (ctl->sslcertpath != NULL)
+		printf(_("  SSL trusted certificate directory: %s\n"), ctl->sslcertpath);
+	}
+	if (ctl->sslfingerprint != NULL)
+		printf(_("  SSL key fingerprint (checked against the server key): %s\n"), ctl->sslfingerprint);
 #endif
 	if (ctl->server.timeout > 0)
 	    printf(_("  Server nonresponse timeout is %d seconds"), ctl->server.timeout);
