@@ -287,6 +287,9 @@ struct query
     char *sslkey;		/* optional SSL private key file */
     char *sslcert;		/* optional SSL certificate file */
 	char *sslproto;		/* force usage of protocol (ssl2|ssl3|tls1) - defaults to ssl23 */
+    char *sslcertpath;		/* Trusted certificate directory for checking the server cert */
+    flag sslcertck;		/* Strictly check the server cert. */
+    char *sslfingerprint;	/* Fingerprint to check against */
     char *properties;		/* passthrough properties for extensions */
 
     /* internal use -- per-poll state */
