@@ -35,7 +35,7 @@ static flag do_idle;
 static char capabilities[MSGBUFSIZE+1];
 static unsigned int *unseen_messages;
 
-int imap_ok(int sock, char *argbuf)
+static int imap_ok(int sock, char *argbuf)
 /* parse command response */
 {
     char buf[MSGBUFSIZE+1];
@@ -83,12 +83,16 @@ int imap_ok(int sock, char *argbuf)
 	 * which causes fetchmail to go into a endless loop
 	 * fetching the same message over and over again. 
 	 * 
+	 * However, for check_only, we use EXAMINE which will
+	 * mark the mailbox read-only as per the RFC.
+	 * 
 	 * This checks for the condition and aborts if 
 	 * the mailbox is read-only. 
 	 *
 	 * See RFC 2060 section 6.3.1 (SELECT).
+	 * See RFC 2060 section 6.3.2 (EXAMINE).
 	 */ 
-	else if (strstr(buf, "[READ-ONLY]"))
+	else if (!check_only && strstr(buf, "[READ-ONLY]"))
 	    return(PS_LOCKBUSY);
     } while
 	(tag[0] != '\0' && strncmp(buf, tag, strlen(tag)));
@@ -206,7 +210,7 @@ static int do_imap_ntlm(int sock, struct query *ctl)
 }
 #endif /* NTLM */
 
-int imap_canonicalize(char *result, char *raw, int maxlen)
+static int imap_canonicalize(char *result, char *raw, int maxlen)
 /* encode an IMAP password as per RFC1730's quoting conventions */
 {
     int i, j;
@@ -223,7 +227,7 @@ int imap_canonicalize(char *result, char *raw, int maxlen)
     return(i);
 }
 
-int imap_getauth(int sock, struct query *ctl, char *greeting)
+static int imap_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
     int ok = 0;
@@ -232,8 +236,10 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     capabilities[0] = '\0';
     if ((ok = gen_transact(sock, "CAPABILITY")) == PS_SUCCESS)
     {
-	/* UW-IMAP server 10.173 notifies in all caps */
-	if (strstr(capabilities, "IMAP4REV1"))
+	/* UW-IMAP server 10.173 notifies in all caps, but RFC2060 says we
+	   should expect a response in mixed-case */
+	if (strstr(capabilities, "IMAP4REV1") ||
+	    strstr(capabilities, "IMAP4rev1"))
 	{
 	    imap_version = IMAP4rev1;
 	    if (outlevel >= O_DEBUG)
@@ -291,23 +297,38 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
      * Time to authenticate the user.
      * Try the protocol variants that don't require passwords first.
      */
+    ok = PS_AUTHFAIL;
+
 #ifdef GSSAPI
     if ((ctl->server.authenticate == A_ANY 
-	 || ctl->server.authenticate==A_GSSAPI)
+	 || ctl->server.authenticate == A_GSSAPI)
 	&& strstr(capabilities, "AUTH=GSSAPI"))
-	return(do_gssauth(sock, ctl->server.truename, ctl->remotename));
+	if(ok = do_gssauth(sock, "AUTHENTICATE", ctl->server.truename, ctl->remotename))
+	{
+	    /* SASL cancellation of authentication */
+	    gen_send(sock, "*");
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return ok;
 #endif /* GSSAPI */
 
 #ifdef KERBEROS_V4
     if ((ctl->server.authenticate == A_ANY 
-	 || ctl->server.authenticate==A_KERBEROS_V4
-	 || ctl->server.authenticate==A_KERBEROS_V5) 
+	 || ctl->server.authenticate == A_KERBEROS_V4
+	 || ctl->server.authenticate == A_KERBEROS_V5) 
 	&& strstr(capabilities, "AUTH=KERBEROS_V4"))
     {
 	if ((ok = do_rfc1731(sock, "AUTHENTICATE", ctl->server.truename)))
+	{
 	    /* SASL cancellation of authentication */
 	    gen_send(sock, "*");
-	return(ok);
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return ok;
     }
 #endif /* KERBEROS_V4 */
 
@@ -317,40 +338,59 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
      */
 
     if ((ctl->server.authenticate == A_ANY 
-	 || ctl->server.authenticate==A_CRAM_MD5)
+	 || ctl->server.authenticate == A_CRAM_MD5)
 	&& strstr(capabilities, "AUTH=CRAM-MD5"))
     {
 	if ((ok = do_cram_md5 (sock, "AUTHENTICATE", ctl)))
+	{
 	    /* SASL cancellation of authentication */
 	    gen_send(sock, "*");
-	return(ok);
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return ok;
     }
 
 #if OPIE_ENABLE
     if ((ctl->server.authenticate == A_ANY 
-	 || ctl->server.authenticate==A_OTP)
+	 || ctl->server.authenticate == A_OTP)
 	&& strstr(capabilities, "AUTH=X-OTP"))
-	return(do_otp(sock, ctl));
+	if ((ok = do_otp(sock, "AUTHENTICATE", ctl)))
+	{
+	    /* SASL cancellation of authentication */
+	    gen_send(sock, "*");
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return ok;
 #else
-    if (ctl->server.authenticate==A_NTLM)
+    if (ctl->server.authenticate == A_NTLM)
     {
 	report(stderr, 
 	   _("Required OTP capability not compiled into fetchmail\n"));
-	return(PS_AUTHFAIL);
     }
 #endif /* OPIE_ENABLE */
 
 #ifdef NTLM_ENABLE
     if ((ctl->server.authenticate == A_ANY 
-	 || ctl->server.authenticate==A_NTLM) 
+	 || ctl->server.authenticate == A_NTLM) 
 	&& strstr (capabilities, "AUTH=NTLM"))
-	return(do_imap_ntlm(sock, ctl));
+	if ((ok = do_imap_ntlm(sock, ctl)))
+	{
+	    /* SASL cancellation of authentication */
+	    gen_send(sock, "*");
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return(ok);
 #else
-    if (ctl->server.authenticate==A_NTLM)
+    if (ctl->server.authenticate == A_NTLM)
     {
 	report(stderr, 
 	   _("Required NTLM capability not compiled into fetchmail\n"));
-	return(PS_AUTHFAIL);
     }
 #endif /* NTLM_ENABLE */
 
@@ -360,24 +400,35 @@ int imap_getauth(int sock, struct query *ctl, char *greeting)
     {
 	report(stderr, 
 	       _("Required LOGIN capability not supported by server\n"));
-	return PS_AUTHFAIL;
     }
 #endif /* __UNUSED__ */
 
     /* we're stuck with sending the password en clair */
+    if ((ctl->server.authenticate == A_ANY 
+ 	 || ctl->server.authenticate == A_PASSWORD) 
+ 	&& !strstr (capabilities, "LOGINDISABLED"))
     {
 	/* these sizes guarantee no buffer overflow */
 	char	remotename[NAMELEN*2+1], password[PASSWORDLEN*2+1];
 
 	imap_canonicalize(remotename, ctl->remotename, NAMELEN);
 	imap_canonicalize(password, ctl->password, PASSWORDLEN);
+
+	strcpy(shroud, ctl->password);
 	ok = gen_transact(sock, "LOGIN \"%s\" \"%s\"", remotename, password);
+	shroud[0] = '\0';
+	if (ok)
+	{
+	    /* SASL cancellation of authentication */
+	    gen_send(sock, "*");
+	    if(ctl->server.authenticate != A_ANY)
+                return ok;
+	}
+	else
+	    return(ok);
     }
 
-    if (ok)
-	return(ok);
-    
-    return(PS_SUCCESS);
+    return(ok);
 }
 
 static int internal_expunge(int sock)
@@ -823,7 +874,6 @@ const static struct method imap =
     TRUE,		/* this is a tagged protocol */
     FALSE,		/* no message delimiter */
     imap_ok,		/* parse command response */
-    imap_canonicalize,	/* deal with embedded slashes and spaces */
     imap_getauth,	/* get authorization */
     imap_getrange,	/* query range of messages */
     imap_getsizes,	/* get sizes of messages (used for ESMTP SIZE option) */
